@@ -1,4 +1,5 @@
 import math
+import pprint
 import sys
 import traceback
 from typing import Any
@@ -10,37 +11,44 @@ from typing import TYPE_CHECKING
 from typing import Text
 from typing import Union
 
-from .compat import StringIO
-from .compat import ensure_text
-from .compat import is_integer
-from .compat import iteritems
-from .compat import numeric_types
-from .compat import stringify
-from .compat import time_ns
+import six
+
+from . import config
+from .constants import ERROR_MSG
+from .constants import ERROR_STACK
+from .constants import ERROR_TYPE
 from .constants import MANUAL_DROP_KEY
 from .constants import MANUAL_KEEP_KEY
 from .constants import NUMERIC_TAGS
 from .constants import SERVICE_KEY
 from .constants import SERVICE_VERSION_KEY
 from .constants import SPAN_MEASURED_KEY
+from .constants import USER_KEEP
+from .constants import USER_REJECT
 from .constants import VERSION_KEY
+from .context import Context
 from .ext import SpanTypes
-from .ext import errors
 from .ext import http
 from .ext import net
-from .ext import priority
 from .internal import _rand
+from .internal.compat import NumericType
+from .internal.compat import StringIO
+from .internal.compat import ensure_text
+from .internal.compat import is_integer
+from .internal.compat import iteritems
+from .internal.compat import numeric_types
+from .internal.compat import stringify
+from .internal.compat import time_ns
 from .internal.logger import get_logger
-from .vendor import six
 
 
 if TYPE_CHECKING:
-    from .context import Context
     from .tracer import Tracer
 
 
-_MetaKeyType = Union[Text, bytes]
-_MetaDictType = Dict[_MetaKeyType, Text]
+_TagNameType = Union[Text, bytes]
+_MetaDictType = Dict[_TagNameType, Text]
+_MetricDictType = Dict[_TagNameType, NumericType]
 
 log = get_logger(__name__)
 
@@ -51,7 +59,7 @@ class Span(object):
         # Public span attributes
         "service",
         "name",
-        "resource",
+        "_resource",
         "span_id",
         "trace_id",
         "parent_id",
@@ -66,6 +74,7 @@ class Span(object):
         "sampled",
         # Internal attributes
         "_context",
+        "_local_root",
         "_parent",
         "_ignored_exceptions",
         "_on_finish_callbacks",
@@ -84,13 +93,16 @@ class Span(object):
         parent_id=None,  # type: Optional[int]
         start=None,  # type: Optional[int]
         context=None,  # type: Optional[Context]
-        on_finish=None,  # type: List[Callable[[Span], None]]
-        _check_pid=True,  # type: bool
+        on_finish=None,  # type: Optional[List[Callable[[Span], None]]]
     ):
         # type: (...) -> None
         """
         Create a new span. Call `finish` once the traced operation is over.
 
+        **Note:** A ``Span`` should only be accessed or modified in the process
+        that it was created in. Using a ``Span`` from within a child process
+        could result in a deadlock or unexpected behavior.
+
         :param ddtrace.Tracer tracer: the tracer that will submit this span when
             finished.
         :param str name: the name of the traced operation.
@@ -118,32 +130,33 @@ class Span(object):
         # required span info
         self.name = name
         self.service = service
-        self.resource = resource or name
+        self._resource = [resource or name]
         self._span_type = None
         self.span_type = span_type
 
         # tags / metadata
         self.meta = {}  # type: _MetaDictType
         self.error = 0
-        self.metrics = {}  # type: Dict[str, Any]
+        self.metrics = {}  # type: _MetricDictType
 
         # timing
-        self.start_ns = time_ns() if start is None else int(start * 1e9)
+        self.start_ns = time_ns() if start is None else int(start * 1e9)  # type: int
         self.duration_ns = None  # type: Optional[int]
 
         # tracing
-        self.trace_id = trace_id or _rand.rand64bits(check_pid=_check_pid)  # type: int
-        self.span_id = span_id or _rand.rand64bits(check_pid=_check_pid)  # type: int
+        self.trace_id = trace_id or _rand.rand64bits()  # type: int
+        self.span_id = span_id or _rand.rand64bits()  # type: int
         self.parent_id = parent_id  # type: Optional[int]
-        self.tracer = tracer
+        self.tracer = tracer  # type: Optional[Tracer]
         self._on_finish_callbacks = [] if on_finish is None else on_finish
 
         # sampling
         self.sampled = True  # type: bool
 
-        self._context = context  # type: Optional[Context]
+        self._context = context._with_span(self) if context else None  # type: Optional[Context]
         self._parent = None  # type: Optional[Span]
         self._ignored_exceptions = None  # type: Optional[List[Exception]]
+        self._local_root = None  # type: Optional[Span]
 
     def _ignore_exception(self, exc):
         # type: (Exception) -> None
@@ -163,6 +176,14 @@ class Span(object):
         # type: (Union[int, float]) -> None
         self.start_ns = int(value * 1e9)
 
+    @property
+    def resource(self):
+        return self._resource[0]
+
+    @resource.setter
+    def resource(self, value):
+        self._resource[0] = value
+
     @property
     def span_type(self):
         return self._span_type
@@ -204,31 +225,24 @@ class Span(object):
         self.duration_ns = int(value * 1e9)
 
     def finish(self, finish_time=None):
-        # type: (Optional[int]) -> None
+        # type: (Optional[float]) -> None
         """Mark the end time of the span and submit it to the tracer.
-        If the span has already been finished don't do anything
+        If the span has already been finished don't do anything.
 
-        :param int finish_time: The end time of the span in seconds.
-                                Defaults to now.
+        :param finish_time: The end time of the span, in seconds. Defaults to ``now``.
         """
-        if self.finished:
+        if self.duration_ns is not None:
             return
 
-        if self.duration_ns is None:
-            ft = time_ns() if finish_time is None else int(finish_time * 1e9)
-            # be defensive so we don't die if start isn't set
-            self.duration_ns = ft - (self.start_ns or ft)
-
-        if self._context:
-            trace, sampled = self._context.close_span(self)
-            if self.tracer and trace and sampled:
-                self.tracer.write(trace)
+        ft = time_ns() if finish_time is None else int(finish_time * 1e9)
+        # be defensive so we don't die if start isn't set
+        self.duration_ns = ft - (self.start_ns or ft)
 
         for cb in self._on_finish_callbacks:
             cb(self)
 
     def set_tag(self, key, value=None):
-        # type: (_MetaKeyType, Any) -> None
+        # type: (_TagNameType, Any) -> None
         """Set a tag key/value pair on the span.
 
         Keys must be strings, values must be ``stringify``-able.
@@ -257,13 +271,13 @@ class Span(object):
         INT_TYPES = (net.TARGET_PORT,)
         if key in INT_TYPES and not val_is_an_int:
             try:
-                value = int(value)  # type: ignore[arg-type]
+                value = int(value)
                 val_is_an_int = True
             except (ValueError, TypeError):
                 pass
 
         # Set integers that are less than equal to 2^53 as metrics
-        if val_is_an_int and abs(value) <= 2 ** 53:  # type: ignore[arg-type]
+        if value is not None and val_is_an_int and abs(value) <= 2 ** 53:
             self.set_metric(key, value)
             return
 
@@ -274,6 +288,10 @@ class Span(object):
 
         # Key should explicitly be converted to a float if needed
         elif key in NUMERIC_TAGS:
+            if value is None:
+                log.debug("ignoring not number metric %s:%s", key, value)
+                return
+
             try:
                 # DEV: `set_metric` will try to cast to `float()` for us
                 self.set_metric(key, value)
@@ -283,10 +301,10 @@ class Span(object):
             return
 
         elif key == MANUAL_KEEP_KEY:
-            self.context.sampling_priority = priority.USER_KEEP
+            self.context.sampling_priority = USER_KEEP
             return
         elif key == MANUAL_DROP_KEY:
-            self.context.sampling_priority = priority.USER_REJECT
+            self.context.sampling_priority = USER_REJECT
             return
         elif key == SERVICE_KEY:
             self.service = value
@@ -310,20 +328,25 @@ class Span(object):
             log.warning("error setting tag %s, ignoring it", key, exc_info=True)
 
     def _set_str_tag(self, key, value):
-        # type: (_MetaKeyType, Text) -> None
+        # type: (_TagNameType, Text) -> None
         """Set a value for a tag. Values are coerced to unicode in Python 2 and
         str in Python 3, with decoding errors in conversion being replaced with
         U+FFFD.
         """
-        self.meta[key] = ensure_text(value, errors="replace")
+        try:
+            self.meta[key] = ensure_text(value, errors="replace")
+        except Exception as e:
+            if config._raise:
+                raise e
+            log.warning("Failed to set text tag '%s'", key, exc_info=True)
 
     def _remove_tag(self, key):
-        # type: (str) -> None
+        # type: (_TagNameType) -> None
         if key in self.meta:
             del self.meta[key]
 
     def get_tag(self, key):
-        # type: (_MetaKeyType) -> Optional[Text]
+        # type: (_TagNameType) -> Optional[Text]
         """Return the given tag or None if it doesn't exist."""
         return self.meta.get(key, None)
 
@@ -337,7 +360,7 @@ class Span(object):
                 self.set_tag(k, v)
 
     def set_meta(self, k, v):
-        # type: (_MetaKeyType, Text) -> None
+        # type: (_TagNameType, NumericType) -> None
         self.set_tag(k, v)
 
     def set_metas(self, kvs):
@@ -345,7 +368,7 @@ class Span(object):
         self.set_tags(kvs)
 
     def set_metric(self, key, value):
-        # type: (str, Any) -> None
+        # type: (_TagNameType, NumericType) -> None
         # This method sets a numeric tag value for the given key. It acts
         # like `set_meta()` and it simply add a tag without further processing.
 
@@ -378,13 +401,13 @@ class Span(object):
         self.metrics[key] = value
 
     def set_metrics(self, metrics):
-        # type: (Dict[str, Any]) -> None
+        # type: (_MetricDictType) -> None
         if metrics:
             for k, v in iteritems(metrics):
                 self.set_metric(k, v)
 
     def get_metric(self, key):
-        # type: (str) -> Any
+        # type: (_TagNameType) -> Optional[NumericType]
         return self.metrics.get(key)
 
     def to_dict(self):
@@ -413,10 +436,10 @@ class Span(object):
             d["duration"] = self.duration_ns
 
         if self.meta:
-            d["meta"] = self.meta  # type: ignore[assignment]
+            d["meta"] = self.meta
 
         if self.metrics:
-            d["metrics"] = self.metrics  # type: ignore[assignment]
+            d["metrics"] = self.metrics
 
         if self.span_type:
             d["type"] = self.span_type
@@ -434,11 +457,11 @@ class Span(object):
             self.set_exc_info(exc_type, exc_val, exc_tb)
         else:
             tb = "".join(traceback.format_stack(limit=limit + 1)[:-1])
-            self.meta[errors.ERROR_STACK] = tb
+            self.meta[ERROR_STACK] = tb
 
     def set_exc_info(self, exc_type, exc_val, exc_tb):
         # type: (Any, Any, Any) -> None
-        """ Tag the span with an error tuple as from `sys.exc_info()`. """
+        """Tag the span with an error tuple as from `sys.exc_info()`."""
         if not (exc_type and exc_val and exc_tb):
             return  # nothing to do
 
@@ -455,22 +478,22 @@ class Span(object):
         # readable version of type (e.g. exceptions.ZeroDivisionError)
         exc_type_str = "%s.%s" % (exc_type.__module__, exc_type.__name__)
 
-        self.meta[errors.ERROR_MSG] = str(exc_val)
-        self.meta[errors.ERROR_TYPE] = exc_type_str
-        self.meta[errors.ERROR_STACK] = tb
+        self.meta[ERROR_MSG] = stringify(exc_val)
+        self.meta[ERROR_TYPE] = exc_type_str
+        self.meta[ERROR_STACK] = tb
 
     def _remove_exc_info(self):
         # type: () -> None
-        """ Remove all exception related information from the span. """
+        """Remove all exception related information from the span."""
         self.error = 0
-        self._remove_tag(errors.ERROR_MSG)
-        self._remove_tag(errors.ERROR_TYPE)
-        self._remove_tag(errors.ERROR_STACK)
+        self._remove_tag(ERROR_MSG)
+        self._remove_tag(ERROR_TYPE)
+        self._remove_tag(ERROR_STACK)
 
     def pprint(self):
         # type: () -> str
-        """ Return a human readable version of the span. """
-        lines = [
+        """Return a human readable version of the span."""
+        data = [
             ("name", self.name),
             ("id", self.span_id),
             ("trace_id", self.trace_id),
@@ -479,22 +502,24 @@ class Span(object):
             ("resource", self.resource),
             ("type", self.span_type),
             ("start", self.start),
-            ("end", "" if not self.duration else self.start + self.duration),
-            ("duration", "%fs" % (self.duration or 0)),
+            ("end", None if not self.duration else self.start + self.duration),
+            ("duration", self.duration),
             ("error", self.error),
-            ("tags", ""),
+            ("tags", dict(sorted(self.meta.items()))),
+            ("metrics", dict(sorted(self.metrics.items()))),
         ]
-
-        lines.extend((" ", "%r:%s" % kv) for kv in sorted(self.meta.items()))
-        return "\n".join("%10s %s" % line for line in lines)
+        return " ".join(
+            # use a large column width to keep pprint output on one line
+            "%s=%s" % (k, pprint.pformat(v, width=1024 ** 2).strip())
+            for (k, v) in data
+        )
 
     @property
     def context(self):
-        """
-        Property that provides access to the ``Context`` associated with this ``Span``.
-        The ``Context`` contains state that propagates from span to span in a
-        larger trace.
-        """
+        # type: () -> Context
+        """Return the trace context for this span."""
+        if self._context is None:
+            self._context = Context(trace_id=self.trace_id, span_id=self.span_id)
         return self._context
 
     def __enter__(self):
